/* ReSCAL - Csp templates
 *
 * Copyright (C) 2022
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/// Available CSP template types
/// Note: CGT_NUMBER is the number of generic templates (not a template)
enum CSP_GENERIC_TEMPLATES {CSP_CUSTOM, CSP_BLOCK, CGT_NUMBER};
#ifdef MODEL_DUN
enum CSP_MODEL_TEMPLATES {CSP_LAYER = CGT_NUMBER, CSP_LAYER_COL, CSP_HBLOCK, CSP_CYLINDER, CSP_CONE, CSP_RCONE, CSP_CONE2, CSP_RWALL, CSP_WAVES_2D, CSP_WAVY_NS_LAYER,
CSP_TRIANGLES, CSP_SRC_CEIL, CSP_SRC_UPWIND_CEIL, CSP_SRC_DISK, CSP_SRC_DISK_CEIL, CSP_SRC_RECT, CSP_SRC_RECT_CEIL, CSP_OUT_EAST, CSP_OUT_NORTH, CSP_OUT_SOUTH, CSP_STICKS, CSP_SMILEY};
#elif defined(MODEL_AVA)
enum CSP_MODEL_TEMPLATES {CSP_SRC_CEIL = CGT_NUMBER, CSP_SRC_DISK_CEIL};
#elif defined(MODEL_CMB) || defined(MODEL_ICB)
enum CSP_MODEL_TEMPLATES {CSP_LAYER = CGT_NUMBER};
#elif defined(MODEL_RIV) || defined(MODEL_D2G)
enum CSP_MODEL_TEMPLATES {CSP_LAYER = CGT_NUMBER, CSP_SLOPE, CSP_CLOSED_BORDER, CSP_OPEN_BORDER, CSP_OPEN_BORDER1, CSP_SRC};
#elif defined(MODEL_LIFE)
enum CSP_MODEL_TEMPLATES {CSP_RANDOM = CGT_NUMBER};
//#else
//enum CSP_TEMPLATES {};
#endif

typedef struct _template{
  int type;
  char *name;
  char *desc;
  int nb_args;
  float *args;
  int flag_add;
} CSP_Template;

#define MAX_TEMPLATES 100 //max number of templates

void params_templates();
void init_template_types();
void parse_template_array();
void init_template_array();
void init_templates();
void init_cell_default(Cell *aux);
void init_cell_from_template(Cell *aux, int i, int j, int k, int id_temp);
void init_cell_final(Cell *aux, int i, int j, int k);
void init_csp_from_template_array();
void restore_template_geom(Cell *pt_csp, float angle, int flag_rm, int flag_tmp);
//void restore_template_geom(Cell *pt_csp, float angle, int flag_bounds);
void templates_usage();

